import 'package:clean_network/clean_network.dart';

/// {@template pagination_mixin}
/// A mixin to handle pagination
/// {@endtemplate}
mixin PaginationMixin {
  /// The current page of the paginated response
  int _currentPage = 1;

  int get currentPage => _currentPage;

  /// The last page of the paginated response
  int _lastPage = 1;

  int get lastPage => _lastPage;

  /// The total number of items in the paginated response
  int _total = 1;

  int get total => _total;

  /// The number of items per page in the paginated response
  int _perPage = 20;

  int get perPage => _perPage;

  /// The current length of the list
  int _currentLength = 1;

  /// reset the pagination
  void resetPagination() {
    _currentPage = 1;
    _lastPage = 1;
    _total = 1;
    _currentLength = 1;
    _perPage = 20;
  }

  /// update the pagination
  void updatePagination(PaginatedResponse response) {
    _currentPage = response.pageInfo.currentPage;
    _lastPage = response.pageInfo.lastPage;
    _total = response.pageInfo.total;
    _perPage = response.pageInfo.perPage;
    _currentLength += response.data.length;
  }

  /// Returns true if the current page is less than the last page and the length of the list is less than the total number of items
  bool get hasNextPage => _currentPage < _lastPage && _currentLength < _total;

  /// returns the next page params
  PageParams get nextParams => PageParams(
        currentPage: _currentPage + 1,
        perPage: _perPage,
        lastPage: _lastPage,
        total: _total,
      );
}
